package student

import (
	"bufio"
	"errors"
	"fmt"
	"ouchn/config"
	"sync"
	//"github.com/syndtr/goleveldb/leveldb"
	"io"
	"os"
	"strings"
	"unicode"

	"github.com/tealeg/xlsx"
)

func ParseHan(str string) string {
	ret := ""
	for _, r := range str {
		if unicode.Is(unicode.Han, r) {
			ret = ret + string(r)
		} else if (r >= 48 && r <= 57) || (r >= 65 && r <= 90) || (r >= 97 && r <= 112) {
			ret = ret + string(r)
		}
	}
	return ret
}
func getTag(questionType string) string {
	if strings.Contains(questionType, "单选题") || strings.Contains(questionType, "单项") {
		return "1"
	}
	if strings.Contains(questionType, "判断题") {
		return "2"
	}
	if strings.Contains(questionType, "多选题") || strings.Contains(questionType, "多项") {
		return "3"
	}
	if strings.Contains(questionType, "计算题") || strings.Contains(questionType, "简答题") {
		return "4"
	}
	return ""
}

var CourseMap map[string]int
var CourseMapArray []map[string][]string
var CourseMapNew map[string]int
var CourseMapArrayNew []map[string][]string
var mutex sync.Mutex
var mutexNew sync.Mutex

func TrimSpace(data string) string {

	data = strings.TrimPrefix(data, "A．")
	data = strings.TrimPrefix(data, "B．")
	data = strings.TrimPrefix(data, "C．")
	data = strings.TrimPrefix(data, "D．")
	str := strings.TrimSpace(data)
	ret := ""
	for _, r := range str {
		if string(r) == " " {
			continue
		}
		if fmt.Sprintf("%x", r) == "a0" {
			continue
		}
		ret = ret + string(r)
	}
	ret = strings.TrimRight(ret, ";")
	ret = strings.TrimRight(ret, "；")
	ret = strings.TrimRight(ret, "。")
	ret = strings.TrimRight(ret, ".")
	return ret
}

func LoadXlsx(filename string) (map[string][]string, error) {
	if config.Conf.NewStyle {
		return LoadXlsxNew(filename)
	}
	mutex.Lock()
	defer mutex.Unlock()
	if true {
		CourseMap = map[string]int{}
		CourseMapArray = []map[string][]string{}
	}
	if CourseMap == nil {
		CourseMap = map[string]int{}
		CourseMapArray = []map[string][]string{}
	}
	if index, ok := CourseMap[filename]; ok {
		return CourseMapArray[index], nil
	}

	ef, err := xlsx.OpenFile(filename)
	if err != nil {
		fmt.Printf("err:%v\n", err)
		return nil, err
	}
	if len(ef.Sheets) == 0 {
		return nil, errors.New("excel empty sheet")
	}
	sheet := ef.Sheets[0]
	rows := sheet.Rows
	if len(rows) == 0 {
		return nil, errors.New("excel empty rows")
	}
	ret := map[string][]string{}
	for i, row := range rows {
		cells := row.Cells
		if len(cells) == 0 {
			continue
		}
		emptyCells := true
		for _, v := range cells {
			if TrimSpace(v.Value) != "" {
				emptyCells = false
				break
			}
		}
		if emptyCells {
			continue
		}

		if len(cells) < 3 {
			return nil, errors.New(fmt.Sprintf("第%d行信息不足", i+1))
		}
		questionType := cells[0].Value
		if questionType == "" {
			break
		}
		tag := getTag(questionType)
		if tag == "" {
			return nil, errors.New(fmt.Sprintf("第%d行题目类型错误", i+1))
		}
		questionName := cells[1].Value
		questionName = TrimSpace(questionName)
		questionName = ParseHan(questionName)
		ret[tag+"-"+questionName] = []string{}

		for i := 2; i < len(cells); i++ {
			answer := cells[i].Value
			answer = TrimSpace(answer)
			if answer == "" {
				continue
			}

			ret[tag+"-"+questionName] = append(ret[tag+"-"+questionName], answer)
		}
	}
	CourseMapArray = append(CourseMapArray, ret)
	CourseMap[filename] = len(CourseMapArray) - 1
	return ret, nil
}

var SelectAnswerFlagMap = map[string]int{
	"A": 0,
	"a": 0,
	"B": 1,
	"b": 1,
	"C": 2,
	"c": 2,
	"D": 3,
	"d": 3,
	"E": 4,
	"e": 4,
	"F": 5,
	"f": 5,
	"G": 6,
	"g": 6,
	"H": 7,
	"h": 7,
	"I": 8,
	"i": 8,
	"J": 9,
	"j": 9,
	"K": 10,
	"k": 10,
	"L": 11,
	"l": 11,
	"M": 12,
	"m": 12,
	"N": 13,
	"n": 13,
}
var JudgeAnswerMap = map[string]int{
	"Y": 0,
	"y": 0,
	"是": 0,
	"N": 1,
	"n": 1,
	"否": 1,
}

func LoadXlsxNew(filename string) (map[string][]string, error) {
	mutexNew.Lock()
	defer mutexNew.Unlock()
	if true {
		CourseMapNew = map[string]int{}
		CourseMapArrayNew = []map[string][]string{}
	}
	if CourseMapNew == nil {
		CourseMapNew = map[string]int{}
		CourseMapArrayNew = []map[string][]string{}
	}
	if index, ok := CourseMapNew[filename]; ok {
		return CourseMapArrayNew[index], nil
	}

	ef, err := xlsx.OpenFile(filename)
	if err != nil {
		fmt.Printf("err:%v\n", err)
		return nil, err
	}
	if len(ef.Sheets) == 0 {
		return nil, errors.New("excel empty sheet")
	}
	sheet := ef.Sheets[0]
	rows := sheet.Rows
	if len(rows) == 0 {
		return nil, errors.New("excel empty rows")
	}
	ret := map[string][]string{}
	for i, row := range rows {
		if i == 0 {
			continue
		}
		cells := row.Cells
		if len(cells) == 0 {
			continue
		}
		emptyCells := true
		for _, v := range cells {
			if TrimSpace(v.Value) != "" {
				emptyCells = false
				break
			}
		}
		if emptyCells {
			continue
		}

		if len(cells) < 5 {
			return nil, errors.New(fmt.Sprintf("第%d行信息不足", i+1))
		}
		questionType := cells[0].Value
		if questionType == "" {
			break
		}
		tag := getTag(questionType)
		if tag == "" {
			return nil, errors.New(fmt.Sprintf("第%d行题目类型错误", i+1))
		}
		questionName := cells[1].Value
		questionName = TrimSpace(questionName)
		questionName = ParseHan(questionName)
		ret[tag+"-"+questionName] = []string{}
		answers := cells[4].Value
		answers = TrimSpace(answers)
		if answers == "" {
			return nil, errors.New(fmt.Sprintf("第%d行答案为空", i+1))
		}

		switch tag {
		case "1":
			ret[tag+"-"+questionName] = append(ret[tag+"-"+questionName], answers)
		case "2":
			ret[tag+"-"+questionName] = append(ret[tag+"-"+questionName], answers)
		case "3":
			array := strings.Split(answers, "|")
			for _, answer := range array {
				if answer == "" {
					continue
				}
				ret[tag+"-"+questionName] = append(ret[tag+"-"+questionName], answer)
			}
		default:
			ret[tag+"-"+questionName] = append(ret[tag+"-"+questionName], answers)
		}

	}
	CourseMapArrayNew = append(CourseMapArrayNew, ret)
	CourseMapNew[filename] = len(CourseMapArrayNew) - 1
	return ret, nil
}

// 从txt 导入 execel
func LoadQuestionFile(filename string) error {

	ef, err := xlsx.OpenFile("国家开放大学学习指南.xlsx")
	if err != nil {
		fmt.Printf("err:%v\n", err)
		return err
	}
	if len(ef.Sheets) == 0 {
		fmt.Printf("err:%v\n", "empty sheet")
		return errors.New("excel empty sheet")
	}
	fmt.Printf("start parse\n")
	sheet := ef.Sheets[0]

	fi, err := os.Open(filename)
	if err != nil {
		return err
	}
	defer fi.Close()

	br := bufio.NewReader(fi)
	tag := 0
	question := ""
	questionOrigin := ""
	for {
		a, _, c := br.ReadLine()
		//fmt.Printf("xx:%s\n", a)
		if c == io.EOF {
			break
		}
		if len(a) == 0 {
			continue
		}
		str := string(a)
		str = strings.TrimSpace(str)

		if str == "" {
			continue
		}
		if strings.Contains(str, "形考") {
			continue
		}

		//fmt.Printf("line:%s\n", line)
		if strings.Contains(str, "单选题") || strings.Contains(str, "单项选") {
			tag = 1
			continue
		}
		if strings.Contains(str, "判断题") {
			tag = 2
			continue
		}

		if tag == 0 {
			continue
		}
		if strings.Contains(str, "题目") {
			continue
		}

		if question == "" {
			question = ParseHan(str)
			questionOrigin = string(a)
			continue
		}

		row := sheet.AddRow()
		cell := row.AddCell()
		cell.Value = "单选题"
		if tag == 2 {
			cell.Value = "判断题"
		}
		cell = row.AddCell()
		cell.Value = questionOrigin
		cell = row.AddCell()
		str = strings.TrimPrefix(str, "正确答案是：")
		str = strings.TrimPrefix(str, "正确答案是:")
		str = strings.TrimPrefix(str, "正确答案是")
		str = strings.TrimPrefix(str, "正确的答案是")
		cell.Value = str
		fmt.Printf("题目：%v\n", question)
		fmt.Printf("答案：%v\n", str)
		question = ""
		//break
	}
	ef.Save("国家开放大学学习指南.xlsx")

	return nil
}
